
(*
 **********************************************
 * Implementazione del cervello del computer
 **********************************************
 *)

unit ClassBrain;

interface

uses
  SysUtils, Math, Poker, ClassCards, PokerEngine;

type
  TBrain = class
  public
    carte : array[1..5] of TCard;
    soldi : Integer;
    function CambioCarte(var pos: array of TPos): Integer;
    function Apertura: Integer;
    function Rilancio: Integer;

  private
    //skillPlayer : integer;

  end;

var
  Compt: TBrain;

implementation


{ TBrain }


//** Per ora apre sempre **//
function TBrain.Apertura: Integer;
var
  punteggio: Cardinal;
begin
  CalcolaPunteggio(carte, punteggio);
  if ultimaPuntata > 0 then
  begin
    if punteggio > _tris then
      result := ultimaPuntata + 200
    else
      result := ultimaPuntata;
  end
  else
    result := 100;
end;

(*
 * Funzione per cambiare le carte del computer
 * Ritorna zero se il computer  servito oppure torna il numero di carte da cambiare
 * In pos ritorna la posizione della carta da cambiare
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
function TBrain.CambioCarte(var pos: array of TPos): Integer;
var
  punteggio: Cardinal;
  cartns: array[1..5] of TCarta;
  cnc: array[1..5] of integer; //** Coincidenze
  i, j, tmp: Integer;
  f, p, q, c: Integer;

const
  _fiori = 1;
  _picche = 2;
  _quadri = 3;
  _cuori = 4;

begin

  //** Azzera l'array coincidenze **//
  for i := 1 to 5 do cnc[i] := 0;

  CalcolaPunteggio(carte, punteggio);

  if punteggio > _SCALA then result := 0
  else
  begin
    for i := 1 to 5 do
      Cards.NumSemFromCard(carte[i], cartns[i].num, cartns[i].sem);

    //** Se ci sono 4 semi uguali tenta colore **//
    f := 0; p := 0; q := 0; c := 0;
    for i := 1 to 5 do
      case cartns[i].sem of
      1: Inc(f);
      2: Inc(p);
      3: Inc(q);
      4: Inc(c);
      end;

    if f = 4 then
      for i := 1 to 5 do
        if cartns[i].sem <> _fiori then
        begin pos[0] := i; result := 1;  Exit end;
    if p = 4 then
      for i := 1 to 5 do
        if cartns[i].sem <> _picche then
        begin pos[0] := i; result := 1;  Exit end;
    if q = 4 then
      for i := 1 to 5 do
        if cartns[i].sem <> _quadri then
        begin pos[0] := i; result := 1;  Exit end;
    if c = 4 then
      for i := 1 to 5 do
        if cartns[i].sem <> _cuori then
        begin pos[0] := i; result := 1;  Exit end;


    //** Se non colore, cambia le carte uniche **//
    result := 0;
    tmp := 0;
    for i := 1 to 5 do
      for j := 1 to 5 do
        if cartns[i].num = cartns[j].num then Inc(cnc[i]);

    //** Se sono tutte diverse, tiene la carta pi alta **//
    j := 0; //** Conta la carta pi alta
    p := 1; //** Conta la posizione
    for i := 1 to 5 do if cnc[i] = 1 then Inc(j);
    if j = 5 then
    begin
      //** Cerca la posizione della carta pi alta **//
      j := cartns[1].num;
      for i := 2 to 5 do
        if j < cartns[i].num then
        begin j := cartns[i].num; p := i end;
      //** Ora in pos[] ci devono essere tutte tranne p **//
      j := -1; //** Pos = 0..4
      for i := 1 to 5 do
        if not(i = p) then
        begin Inc(j); pos[j] := i end;
      result := 4;
      Exit
    end
    else
      for i := 1 to 5 do
        if cnc[i] = 1 then
        begin
          Inc(tmp);
          pos[tmp-1] := i; //** la posizione della carta unica
          Inc(result);
        end;
  end;



end;

(*
 *
 *
 * * * * * * * * * * *)
function TBrain.Rilancio: Integer;
var
  punteggio: Cardinal;
begin
  //result := 300;
  CalcolaPunteggio(carte, punteggio);
  if punteggio < _coppia then  //** debole
  begin
    if ultimaPuntata = 0 then
      result := 100
    else
      if (random(2) + 1) = 1 then //** 1..2
        result := 100
      else
        result := 0;
  end
  else
    if punteggio > _tris then //** forte
    begin
      if ultimaPuntata > 0 then
      begin
        if (random(2) + 1) = 1 then
          result := ultimaPuntata + 500
        else
          result := ultimaPuntata;
      end
      else
        result := 300;
  end
  else //** medio
    if ultimaPuntata > 0 then
    begin
      if (random(2) + 1) = 1 then
        result := ultimaPuntata + 200
      else
        result := ultimaPuntata;
    end
    else
      result := 100;

end;

initialization
  Compt := TBrain.Create;

finalization
  FreeAndNil(Compt);

end.
