

unit ClassCards;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Math;

type

  TPos = 1..5;
  TCard = 1..52;
  TCardNum = 1..13;
  TCardSem = 1..4;

  CCards = class
  public
    constructor Create;
    destructor Destroy; override;

    function UnaCarta(pos: TPos; player: integer): TCard;
    procedure MostraUnaCarta(pos: TPos; carta: TCard; player: Integer);

    procedure MischiaMazzo;

    procedure NumSemFromCard(carta: TCard;
                                var num: TCardNum; var sem: TCardSem);
    function CardToString(num: TCardNum; sem: TCardSem): string;
    function NumCardToString(num: TCardNum): string;
    function NumSemToString(num: TCardSem): string;

  private
    CardWidth:integer;
    CardHeight:integer;

    mazzo: array[1..52] of TCard;
    puntatore : TCard;

  end;






//------------------------------------------------------------------------------


implementation


uses
  Poker, PokerEngine;


//** Costanti per CARDS32.DLL
Const
  //** CdtDraw/Ext, nDraw Field
  C_FACES  = 0;
  C_BACKS  = 1;
  C_INVERT = 2;
  //** Special Cards (C_BACKS)
  CrossHatch  = 53;
  Weave1  = 54;
  Weave2  = 55;
  Robot   = 56;
  Flowers = 57;
  Vine1   = 58;
  Vine2   = 59;
  Fish1   = 60;
  Fish2   = 61;
  Shells  = 62;
  Castle  = 63;
  Island  = 64;
  CardHand= 65;
  UNUSED  = 66;
  TheX    = 67;
  TheO    = 68;


//** Init DLL, Return Cards Size (pixels)
//------------------------------------------------------------------------------
function CdtInit(Var Width,Height: longint): longint;
stdcall; external 'CARDS32.DLL';


//** Draw one of the 68(-1) card at X,Y, Color=highlight color (?)
//------------------------------------------------------------------------------
function CdtDraw(DC: HDC; X,Y,Card,Flag: integer; Color: TColor): integer;
stdcall; external 'CARDS32.DLL';

//** Same as above + draw Width & Height
//------------------------------------------------------------------------------
function CdtDrawExt(DC: HDC;
                    X, Y, W, H, Card, Flag: integer;
                    Color: TColor): integer;
stdcall; external 'CARDS32.DLL';

//** Done
//------------------------------------------------------------------------------
function CdtTerm : integer;
external 'CARDS32.DLL';


//** Fine dichiarazioni per DLL - Inizio dichiarazioni classe CCards **//
//*********************************************************************//

//** Costruttore
//------------------------------------------------------------------------------

constructor CCards.Create;
var
  i : integer;
begin
  inherited Create;

  //** Inizializza la DLL **//
  CdtInit(Cardwidth, CardHeight);

  //** Inizializza il mazzo **//
  for i := 1 to 52 do mazzo[i] := i;

  //** Inizializza il generatore di numeri casuali **//
  RandSeed := GetTickCount;

  //** Mischia e setta il puntatore **//
  MischiaMazzo;

end;

//** Distruttore overraidato
//------------------------------------------------------------------------------

destructor CCards.Destroy;
begin
  CdtTerm;
  inherited Destroy;
end;


//** Ritorna una carta e mostra la carta nel form
//------------------------------------------------------------------------------

function CCards.UnaCarta(pos: TPos; player: Integer): TCard;
begin
  case player of
  _PLAYER:
    CdtDraw ( Form1.Canvas.Handle,
              15+cardWidth * (pos - 1),
              player,
              mazzo[puntatore],
              C_FACES,
              0 );
  _COMPUTER:
    CdtDraw ( Form1.Canvas.Handle,
              15+cardWidth * (pos - 1),
              player,
              robot, //card's back
              C_BACKS,
              0 );
  end;



  result := mazzo[puntatore];
  Inc(puntatore);
end;


//** Visualizza una carta **//
//------------------------------------------------------------------------------

procedure CCards.MostraUnaCarta(pos: TPos; carta: TCard; player: Integer);
begin
  CdtDraw ( Form1.Canvas.Handle,
            15+cardWidth * (pos - 1),
            player,
            carta,
            C_FACES,
            0 );
end;

//** Mischia il mazzo
//------------------------------------------------------------------------------

procedure CCards.MischiaMazzo;
var
  i : integer;
  c1, c2, tmp : TCard;
begin
  for i := 1 to random(100) + 100 do  //** da 100 a 199 mischiate
  begin
    c1 := random(52) + 1;             //** da 1 a 52
    c2 := random(52) + 1;
    tmp := mazzo[c1];                 //** Esegue lo swap
    mazzo[c1] := mazzo[c2];
    mazzo[c2] := tmp;
  end;
  puntatore := 1;
end;

//** Calcola numero e seme dalla carta
//------------------------------------------------------------------------------
procedure CCards.NumSemFromCard(carta: TCard;
                         var num: TCardNum; var sem: TCardSem);
begin
  sem := Ceil(carta / 13);
  // ** Adatta il numero ai valori per poker
  case sem of
    2: sem := 3;
    3: sem := 4;
    4: sem := 2;
  end;
  num := carta;
  while num > 13 do Dec(num, 13);
  //** adatta il numero ai valori del poker
  if num = 1 then num := 13 else dec(num);
end;

//** Ritorna il numero e il seme della carta in stringa
//------------------------------------------------------------------------------
function CCards.CardToString(num: TCardNum; sem: TCardSem) : string;
begin
  result := NumCardToString(num) + ' di ' + NumSemToString(sem);
end;


function CCards.NumCardToString(num: TCardNum) : string;
begin
  case num of
  1: result := 'Due';
  2: result := 'Tre';
  3: result := 'Quattro';
  4: result := 'Cinque';
  5: result := 'Sei';
  6: result := 'Sette';
  7: result := 'Otto';
  8: result := 'Nove';
  9: result := 'Dieci';
  10: result := 'Jack';
  11: result := 'Donna';
  12: result := 'Re';
  13: result := 'Asso';
  end;
end;

function CCards.NumSemToString(num: TCardSem) : string;
begin
  case num of
    1: result := 'Fiori';
    2: result := 'Picche';
    3: result := 'Denari';
    4: result := 'Cuori';
  end;
end;


initialization
  Cards := CCards.Create;

finalization
  Cards.Free;


end.
